      Subroutine DipolGammaRij (iqed, E, theta, A, B, R)

c              For  gamma gamma -> tau- tau+  reaction
c
c  Calculates spin-correlation coefficients R(i,j) as functions of beam
c  energy E = sqrt(s)/2 (in GeV) and scattaring angle theta.
c  V is velocity of tau lepton, gam is Lorentz factor, alpha is
c  fine-structure constant.
c  Anomalous magnetic moment A1 = ASM + A and electric dipole moment B are
c  real constants.
c  Parameters A, B describe 'New Physics'.
c  ASM is anomalous magnetic moment of tau in Standard Model (SM):
c  S.Eidelman and M.Passera. Mod.Phys.Lett. A22, 159-179, 2007.
c
c  Order of coefficients:
c     i = 1,2,3 correspond to S_x, S_y, S_z for tau-,
c     j = 1,2,3 correspond to S'_x, S'_y, S'_z for tau+
c     i,j = 4 (equivalent to tt) corresponds to 1 (no spin dependence).

      Implicit none
      integer iqed
      integer i, j                         ! not used here
      real*8 E, theta, A, B, ASM, A1, gam, e4, V
      real*8 R(1:4, 1:4)
      real*8 PI/3.141592653589793238d0/,alpha/7.2973525693d-3/
      real*8 mtau/1.77686d0/               ! mass of tau in GeV

      V = dsqrt(1.d0 -(mtau/E)**2)         ! tau velocity 
      e4 = (4.0d0 *PI *alpha)**2           ! (electric charge)^4
      gam = E/mtau                         ! Lorentz factor for tau

c  Cotribution to anomalous magnetic moment in Standard Model:
      ASM = 1.17721d-3
      ASM = ASM *iqed                     ! switch for dipole SM part
      A1 = ASM + A                        ! Standard Model + New Physics

      D2 = (V**2 *dcos(theta)**2 -1.d0)**2        ! factor in denominator

      R(1,1) = e4 *(-11.d0 *V**4 +28.d0 *A1 *V**2 +
     $ 4.d0 *(V**2 -2.d0) *dcos(2.d0*theta) *V**2 -
     $ (V**2 -4.d0 *A1 -2.d0) *dcos(4.d0*theta) *V**2 +
     $ 22.d0 *V**2 -32.d0 *A1 -8.d0) /(8.d0 *D2)

      R(1,2) = e4 *V *B *(dcos(4.d0*theta) *V**2 +15.d0 *V**2 +
     $ 4.d0 *cos(2.d0*theta) -20.d0) /(4.d0 *D2)

      R(1,3) = e4 *V**2 *gam *((A1 -1.d0) *V**2 +
     $ (V**2 +A1 *(V**2 -2.d0) -1.d0) *dcos(2.d0*theta) +1.d0) *
     $ dsin(2.d0*theta) /(2.d0 *D2)

      R(1,4) = 0.d0

      R(2,1) = -R(1,2)

      R(2,2) = e4 *(-dcos(4.d0*theta) *V**4 -11.d0 *V**4 +
     $ 16.d0 *A1 *V**2 +4.d0 *(V**2 +4.d0 *A1) *dcos(2.d0*theta) *V**2 +
     $ 16.d0 *V**2 -32.d0 *A1 -8.d0) /(8.d0 *D2)

      R(2,3) = e4 *V *gam *B *(dcos(2.d0*theta)*V**2 -3.d0*V**2 +2.d0) *
     $ dsin(2.d0*theta) /(2.d0 *D2)

      R(2,4) = 0.d0

      R(3,1) = R(1,3)

      R(3,2) = -R(2,3)

      R(3,3) = e4 *(-4.d0 *dcos(2.d0*theta) *V**4 +11.d0 *V**4 +
     $ 36.d0 *A1 *V**2 +(V**2 -4.d0 *A1 -2.d0) *dcos(4.d0*theta) *V**2 +
     $ 2.d0 *V**2 -32.d0 *A1 -8.d0) /(8.d0 *D2)

      R(3,4) = 0.d0

      R(4,1) = 0.d0

      R(4,2) = 0.d0

      R(4,3) = 0.d0

      R(4,4) = e4 *(-dcos(4.d0*theta) *V**4 -11.d0 *V**4 -16.d0 *A1 *V**2 +
     $ 4.d0 *(V**2 -4.d0 *A1 -2.d0) *dcos(2.d0*theta) *V**2 +
     $ 8.d0 * V**2 +32.d0 *A1 +8.d0) /(8.d0 *D2)

      return
      end